@@ -15,7 +15,11 @@
 
 import time, os
 from collections import OrderedDict
-from xml.etree import ElementTree as ET
+from datetime import datetime
+
+import xml.etree.ElementTree as ET
+
+import xmltodict
 
 from exceptions import InteropFileNotFoundError
 from utils import select_file_from_aliases
@@ -26,8 +30,11 @@ class InteropMetadata(object):
 
     CHANGES:
         
-        0.2     cleaner logical process for using the various XML files. No longer throws exceptions.
-        0.1     first released version.
+        0.3 (in progress) Switching to xmltodict from ElementTree.
+        0.2.2   runParameters supports both MiSeq and HiSeq formats.
+        0.2.1   No longer prioritizing CompletedJobInfo.xml (not reliably present).
+        0.2     Cleaner logical process for using the various XML files. No longer throws exceptions.
+        0.1     First released version.
     """
     
     __version = 0.2     # version of this parser.
@@ -45,9 +52,17 @@ class InteropMetadata(object):
         self.experiment_name = ""        # "RU1453:::/locus/data/run_data//1337/1453"
         self.investigator_name = ""      # "Locus:::Uncle_Jesse - 612 - MiSeq"
         self.runID = ""                  # cf CompletedJobInfo.xml / RTARunInfo / Run param "Id"
-        self.start_datetime = None
+        
+        # TODO: xml_datetimes
+        # We can learn end_datetime this from the RTAComplete.txt file. 
+        #    sample:  2/11/2014,17:25:13.217,Illumina RTA 1.18.42 
+        #
+        #...but it would be nicer if we didn't have to (more files to track, no fun).
+        
+        self.start_datetime = None        
         self.end_datetime = None
-        self.rta_run_info = { }
+        
+        self.rta_run_info = { 'flowcell': '', 'instrument': '', 'date': '' }
 
         # read_config: a list of dictionaries, each of which describe a single read from the sequencer.
         self.read_config = []
@@ -64,9 +79,13 @@ class InteropMetadata(object):
         if self.get_xml_path('reseqstats') is not None:
             self.parse_ResequencingRunStats(self.get_xml_path('reseqstats'))
         
-        # CompletedJobInfo.xml is the most complete XML file in the set for what we need,
-        # However, only RunInfo.xml and/or RunParameters.xml will be available during an active run.
-        # Getting read_config and flowcell_layout are top priority since they inform the binary parsers.
+        # The main goal of parsing the XML is to find out read_config and flowcell_layout.
+        # A lot of other data is available, but only these two basics are necessary.
+        #
+        # CompletedJobInfo.xml has the most complete data from MiSeq machines, but only exists
+        # at the end of a run, and HiSeq machines don't even generate one.
+        #
+        # RunInfo.xml (containing just the basics) is always available during an active run.
         #
         # TODO: xml_flex (proposed improvement allowing a config file to set which tokens are required / not required.)
         #       Also we might want to specify priority of provenance (e.g. get start_datetime from 'runparams' first).
@@ -85,7 +104,8 @@ class InteropMetadata(object):
                     break
 
     def _set_xml_map(self):
-        "finds all available XML files and assigns them to an ordered dictionary mapping of codename:[filepath,parse_function]"
+        """finds all available XML files, assigns them to an ordered dictionary 
+        mapping of codename:[filepath,parse_function] """
         for codename in self._xml_map:
             self._xml_map[codename][0] = self.get_xml_path(codename)
                 
@@ -97,11 +117,11 @@ class InteropMetadata(object):
     def parse_Run_ET(self, run_ET):
         "parses chunk of XML associated with the RTA Run Info blocks in (at least) 2 xml files."
 
-        self.rta_run_info = { 'instrument': run_ET.find("Instrument").text,     # M00612
-                              'flowcell': run_ET.find("Flowcell").text,         # 000000000-A316T
-                              'date': run_ET.find("Date").text }                # 130208
+        self.rta_run_info = { 'instrument': run_ET.find('Instrument').text,     # M00612
+                              'flowcell': run_ET.find('Flowcell').text,         # 000000000-A316T
+                              'date': run_ET.find('Date').text }                # 130208
     
-        flowcell_ET = run_ET.find("FlowcellLayout")
+        flowcell_ET = run_ET.find('FlowcellLayout')
         self.flowcell_layout = { 'lanecount': int(flowcell_ET.attrib['LaneCount']),
                         'surfacecount': int(flowcell_ET.attrib['SurfaceCount']),
                         'swathcount': int(flowcell_ET.attrib['SwathCount']),
@@ -118,28 +138,91 @@ class InteropMetadata(object):
             read_num += 1       # redundant safety assignment to read_num
             self.read_config.append( {'read_num': read_num,
                                       'cycles': int(item.attrib['NumCycles']), 
-                                      'is_index': True if item.attrib["IsIndexedRead"]=="Y" else False } )      
+                                      'is_index': True if item.attrib['IsIndexedRead']=='Y' else False } )      
     
-    def parse_ResequencingRunStats(self, filepath):
+    def parse_ResequencingRunStats(self, filepath):    
         """Parses ResequencingRunStatistics.xml (or viable alias) to fill instance variables."""
         
+        # TODO: xmltodict conversion 
+
         tree = ET.parse(filepath)
         root = tree.getroot()   # should be "StatisticsResequencing"
         runstats_ET = root.find("RunStats")
         
-        self.resequencing_stats = { 'clusters_raw': int(runstats_ET.find("NumberOfClustersRaw").text),
-                                    'clusters_pf': int(runstats_ET.find("NumberOfClustersPF").text),
-                                    'unindexed': int(runstats_ET.find("NumberOfUnindexedClusters").text),
-                                    'unindexed_pf': int(runstats_ET.find("NumberOfUnindexedClustersPF").text),
-                                    'unaligned': int(runstats_ET.find("NumberOfUnalignedClusters").text),
-                                    'unaligned_pf': int(runstats_ET.find("NumberOfUnalignedClustersPF").text),
-                                    'duplicate': int(runstats_ET.find("NumberOfDuplicateClusters").text) }
+        self.resequencing_stats = { 'clusters_raw': int(runstats_ET.find('NumberOfClustersRaw').text),
+                                    'clusters_pf': int(runstats_ET.find('NumberOfClustersPF').text),
+                                    'unindexed': int(runstats_ET.find('NumberOfUnindexedClusters').text),
+                                    'unindexed_pf': int(runstats_ET.find('NumberOfUnindexedClustersPF').text),
+                                    'unaligned': int(runstats_ET.find('NumberOfUnalignedClusters').text),
+                                    'unaligned_pf': int(runstats_ET.find('NumberOfUnalignedClustersPF').text),
+                                    'duplicate': int(runstats_ET.find('NumberOfDuplicateClusters').text) }
 
+    def parse_RunInfo(self, filepath):
+        "parses Reads, Date, Flowcell, Instrument out of runInfo.xml"
+        
+        #buf = open(filepath).read()
+        #root = xmltodict.parse(buf)['RunInfo']
+        
+        tree = ET.parse(filepath)
+        run_ET = tree.getroot().find('Run')     #little of use in this file except <Run> subelement. 
+        
+        self.runID = run_ET.attrib['Id']
+        
+        #? is this useful information? if so, what for?  (It's not in CompletedJobInfo.xml)
+        #self.runNumber = run_ET.attrib['Number'] 
+        self.rta_run_info = self.parse_Run_ET(run_ET)
+
+
+    def _parse_runparams(self, xml_dict):
+        # Different format from that in CompletedJobInfo.xml (contains read Number).
+        # And there are two possible keys to indicate the same datastructure. So fun.
+        try:
+            Reads = xml_dict.get('Reads')['Read']
+        except KeyError:
+            Reads = xml_dict.get('Reads')['RunInfoRead']
+
+        for read in Reads:
+            self.read_config.append( 
+                    {'read_num': int(read['@Number']),
+                     'cycles': int(read['@NumCycles']),
+                     'is_index': True if read['@IsIndexedRead']=='Y' else False 
+                    } )
+
+        self.rta_version = xml_dict.get('RTAVersion', '')
+        
+        rawdate = xml_dict.get('RunStartDate', '')    # format: 130208 YYMMDD
+        if rawdate: 
+            self.start_datetime = datetime.strptime(rawdate, '%y%m%d')
+            
+        self.runID = xml_dict.get('RunID', '')
+        self.experiment_name = xml_dict.get('ExperimentName', '')        
+        
+
+    def parse_RunParameters(self, filepath):
+        """parses runParameters.xml (or viable alias) to fill instance variables.
+
+        Need to implement further since HiSeq output has no CompletedJobInfo.xml
+        """
+        buf = open(filepath).read()
+        root = xmltodict.parse(buf)['RunParameters']
+
+        self.read_config = []
+        
+        # a dirty hack to figure out which version of this file we're reading.
+        if 'Reads' in list(root['Setup'].keys()):
+            self._parse_runparams(root['Setup'])        # HiSeq
+        else:
+            self._parse_runparams(root)                 # MiSeq
+        
     def parse_CompletedJobInfo(self, filepath):
-        """parses CompletedJobInfo.xml (or viable alias) to fill instance variables."""
+        """parses CompletedJobInfo.xml (or viable alias) to fill instance variables.
+        
+        Not all machines generate this file, so we avoid relying on it.
+        """
+
+        # TODO: xmltodict conversion 
 
         # comments show example data from a real MiSeq run (2013/02)
-        
         tree = ET.parse(filepath)
         root = tree.getroot()       #should be "AnalysisJobInfo"
     
@@ -150,7 +233,7 @@ class InteropMetadata(object):
         # original location of data output from the sequencer.
         self.output_folder = root.find("RTAOutputFolder").text 
         
-        # TODO: reformat these dates into something nicer (more useable).
+        # TODO: xml_datetimes
         self.start_datetime = root.find("StartTime").text           # 2013-02-09T15:51:50.0811937-08:00
         self.end_datetime = root.find("CompletionTime").text        # 2013-02-09T16:06:44.0124452-08:00
         
@@ -160,6 +243,7 @@ class InteropMetadata(object):
         run_ET = root.find("RTARunInfo").find("Run")
         
         # Sheet / *
+        # TODO: deprecate this attribute (can't get it from HiSeq XML)
         try:
             self.runtype = sheet_ET.find("Type").text       # MiSeq, HiSeq, etc.
         except AttributeError:
@@ -167,7 +251,6 @@ class InteropMetadata(object):
             self.runtype = ""
                 
         # Sheet / Header / *
-        # TODO: xml_flex
         try:
             self.investigator_name = header_ET.find("InvestigatorName").text
             self.project_name = header_ET.find("ProjectName").text
@@ -178,42 +261,6 @@ class InteropMetadata(object):
         # RTARunInfo / Run / *          
         self.runID = run_ET.attrib["Id"] 
         self.parse_Run_ET(run_ET)
-
-    def parse_RunParameters(self, filepath):
-        """partially implemented, not essential. Can fill read_config but not flowcell_layout."""
-
-        tree = ET.parse(filepath)
-        root = tree.getroot()
-
-        # TODO: xml_flex - designate source priority for tokens found in multiple sources. 
-        #self.runID = root.find("RunID").text
-        
-        # TODO: normalize this variable
-        #self.start_datetime = root.find('RunStartDate').text    # format: 130208 YYMMDD
-        try:
-            self.experiment_name = root.find('ExperimentName').text
-        except AttributeError:
-            pass
-
-        self.read_config = []
-        for item in root.find('Reads'):
-            #Different format from that in CompletedJobInfo.xml (contains read Number)
-            self.read_config.append( {'read_num': int(item.attrib['Number']),
-                                      'cycles': int(item.attrib['NumCycles']), 
-                                      'is_index': True if item.attrib['IsIndexedRead']=='Y' else False } )
-
-
-    def parse_RunInfo(self, filepath):
-        "parses Reads, Date, Flowcell, Instrument out of runInfo.xml"
-        tree = ET.parse(filepath)
-        run_ET = tree.getroot().find('Run')     #little of use in this file except <Run> subelement. 
-        
-        self.runID = run_ET.attrib['Id']
-        
-        #? is this useful information? if so, what for?  (It's not in CompletedJobInfo.xml)
-        #self.runNumber = run_ET.attrib['Number']    
-        
-        self.rta_run_info = self.parse_Run_ET(run_ET)
         
     def prettyprint_read_config(self):
         out = "Read Config:"
@@ -236,4 +283,3 @@ class InteropMetadata(object):
         out += self.prettyprint_flowcell_layout() + "\n"
         return out
 
-
